package cn.com.qmth.examcloud.core.basic.api.controller;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import javax.persistence.criteria.Predicate;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.fileupload.FileItem;
import org.apache.commons.fileupload.disk.DiskFileItem;
import org.apache.commons.io.FileUtils;
import org.apache.commons.io.IOUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.data.domain.Sort;
import org.springframework.data.domain.Sort.Direction;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.commons.CommonsMultipartFile;

import com.google.common.collect.Lists;
import com.google.common.collect.Maps;

import cn.com.qmth.examcloud.commons.base.exception.StatusException;
import cn.com.qmth.examcloud.commons.base.helpers.DynamicEnum;
import cn.com.qmth.examcloud.commons.base.helpers.DynamicEnumManager;
import cn.com.qmth.examcloud.commons.base.util.PathUtil;
import cn.com.qmth.examcloud.commons.base.util.PropertiesUtil;
import cn.com.qmth.examcloud.commons.base.util.RegExpUtil;
import cn.com.qmth.examcloud.commons.web.helpers.page.PageInfo;
import cn.com.qmth.examcloud.commons.web.security.bean.User;
import cn.com.qmth.examcloud.commons.web.support.ControllerSupport;
import cn.com.qmth.examcloud.core.basic.api.controller.bean.OrgDomain;
import cn.com.qmth.examcloud.core.basic.base.constants.PropKeys;
import cn.com.qmth.examcloud.core.basic.dao.OrgPropertyRepo;
import cn.com.qmth.examcloud.core.basic.dao.OrgRepo;
import cn.com.qmth.examcloud.core.basic.dao.entity.OrgEntity;
import cn.com.qmth.examcloud.core.basic.dao.entity.OrgPropertyEntity;
import cn.com.qmth.examcloud.core.basic.dao.enums.OrgProperty;
import cn.com.qmth.examcloud.core.basic.service.bean.OrgInfo;
import cn.com.qmth.examcloud.core.basic.service.impl.OrgServiceImpl;
import io.swagger.annotations.ApiOperation;

/**
 * 机构服务API Created by songyue on 17/1/14.
 */
@Transactional
@RestController
@RequestMapping("${$rmp.ctr.basic}/org")
public class OrgController extends ControllerSupport {

	@Autowired
	OrgRepo orgRepo;

	@Autowired
	OrgServiceImpl orgService;

	@Autowired
	OrgPropertyRepo orgPropertyRepo;

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param curPage
	 * @param pageSize
	 * @param code
	 * @param domainName
	 * @param name
	 * @return
	 */
	@ApiOperation(value = "分页查询顶级机构")
	@GetMapping("rootOrgPage/{curPage}/{pageSize}")
	public PageInfo<OrgDomain> getRootOrgPage(@PathVariable Integer curPage,
			@PathVariable Integer pageSize, @RequestParam(required = false) String code,
			@RequestParam(required = false) String domainName,
			@RequestParam(required = false) String name,
			@RequestParam(required = false) String propertyKeys) {

		Pageable pageable = new PageRequest(curPage, pageSize, Sort.Direction.DESC, "updateTime");

		Specification<OrgEntity> specification = (root, query, cb) -> {
			List<Predicate> predicates = new ArrayList<>();
			predicates.add(cb.isNull(root.get("parentId")));
			if (StringUtils.isNotEmpty(name)) {
				predicates.add(cb.like(root.get("name"), toSqlSearchPattern(name)));
			}
			if (StringUtils.isNotEmpty(code)) {
				predicates.add(cb.like(root.get("code"), toSqlSearchPattern(code)));
			}
			if (StringUtils.isNotEmpty(domainName)) {
				predicates.add(cb.like(root.get("domainName"), toSqlSearchPattern(domainName)));
			}
			return cb.and(predicates.toArray(new Predicate[predicates.size()]));
		};

		Page<OrgEntity> page = orgRepo.findAll(specification, pageable);
		List<OrgDomain> list = Lists.newArrayList();
		Iterator<OrgEntity> iterator = page.iterator();

		List<String> propertyKeyList = null;
		if (StringUtils.isNotBlank(propertyKeys)) {
			propertyKeyList = RegExpUtil.findAll(propertyKeys, "\\w+");
		}

		while (iterator.hasNext()) {
			OrgEntity next = iterator.next();
			OrgDomain d = new OrgDomain();
			list.add(d);

			d.setCode(next.getCode());
			d.setContacts(next.getContacts());
			d.setEnable(next.getEnable());
			d.setId(next.getId());
			d.setName(next.getName());
			d.setParentId(next.getParentId());
			d.setRemark(next.getRemark());
			d.setRootId(next.getRootId());
			d.setTelephone(next.getTelephone());
			d.setDomainName(next.getDomainName());
			d.setCreationTime(next.getCreationTime());
			d.setUpdateTime(next.getUpdateTime());

			if (CollectionUtils.isNotEmpty(propertyKeyList)) {
				Map<String, String> properties = getProperties(d.getId(), propertyKeyList);
				d.setProperties(properties);
			}
		}

		PageInfo<OrgDomain> ret = new PageInfo<OrgDomain>();
		ret.setList(list);
		ret.setTotal(page.getTotalElements());
		return ret;
	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param orgId
	 * @param propertyKeys
	 * @return
	 */
	private Map<String, String> getProperties(Long orgId, List<String> propertyKeys) {
		DynamicEnumManager manager = OrgProperty.getDynamicEnumManager();
		Map<String, String> map = Maps.newHashMap();
		for (String key : propertyKeys) {
			DynamicEnum de = manager.getByName(key);
			OrgPropertyEntity one = orgPropertyRepo.findByOrgIdAndKeyId(orgId, de.getId());
			if (null != one) {
				map.put(key, one.getValue());
			}
		}

		return map;
	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param rootOrgId
	 * @return
	 */
	@ApiOperation(value = "按ID查询顶级机构", notes = "ID查询")
	@GetMapping("rootOrg/{rootOrgId}")
	public OrgEntity getRootOrg(@PathVariable Long rootOrgId) {
		OrgEntity orgEntity = orgRepo.findOne(rootOrgId);
		if (null == orgEntity) {
			throw new StatusException("B-001250", "rootOrgId is wrong");
		}
		if (null != orgEntity.getParentId()) {
			throw new StatusException("B-001251", "rootOrgId is wrong");
		}
		validateRootOrgIsolation(orgEntity.getRootId());

		return orgEntity;
	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param orgId
	 * @return
	 */
	@ApiOperation(value = "按ID查询子机构", notes = "ID查询")
	@GetMapping("subOrg/{orgId}")
	public OrgEntity getSubOrg(@PathVariable Long orgId) {
		OrgEntity orgEntity = orgRepo.findOne(orgId);
		if (null == orgEntity) {
			throw new StatusException("B-001250", "orgId is wrong");
		}
		if (null == orgEntity.getParentId()) {
			throw new StatusException("B-001251", "orgId is wrong");
		}
		validateRootOrgIsolation(orgEntity.getRootId());

		return orgEntity;
	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param code
	 * @return
	 */
	@ApiOperation(value = "查询顶级机构")
	@GetMapping("getRootOrgByCode")
	public OrgEntity getRootOrgByCode(@RequestParam("code") String code) {
		OrgEntity org = orgRepo.findByParentIdIsNullAndCode(code);
		return org;
	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param curPage
	 * @param pageSize
	 * @param parentId
	 * @param code
	 * @param name
	 * @return
	 */
	@ApiOperation(value = "分页查询子机构")
	@GetMapping("subOrgPage/{curPage}/{pageSize}")
	public PageInfo<OrgDomain> getSubOrgPage(@PathVariable Integer curPage,
			@PathVariable Integer pageSize, @RequestParam(required = true) Long parentId,
			@RequestParam(required = false) String code,
			@RequestParam(required = false) String name,
			@RequestParam(required = false) String propertyKeys) {

		if (null == parentId) {
			throw new StatusException("B-001249", "parentId is null");
		}

		OrgEntity parentOrg = orgRepo.findOne(parentId);
		if (null == parentOrg) {
			throw new StatusException("B-001250", "parentId is wrong");
		}
		validateRootOrgIsolation(parentOrg.getRootId());

		Pageable pageable = new PageRequest(curPage, pageSize, Sort.Direction.DESC, "updateTime");

		Specification<OrgEntity> specification = (root, query, cb) -> {
			List<Predicate> predicates = new ArrayList<>();
			predicates.add(cb.equal(root.get("parentId"), parentId));

			if (StringUtils.isNotEmpty(name)) {
				predicates.add(cb.like(root.get("name"), toSqlSearchPattern(name)));
			}
			if (StringUtils.isNotEmpty(code)) {
				predicates.add(cb.like(root.get("code"), toSqlSearchPattern(code)));
			}
			return cb.and(predicates.toArray(new Predicate[predicates.size()]));
		};

		Page<OrgEntity> page = orgRepo.findAll(specification, pageable);
		List<OrgDomain> list = Lists.newArrayList();
		Iterator<OrgEntity> iterator = page.iterator();

		List<String> propertyKeyList = null;
		if (StringUtils.isNotBlank(propertyKeys)) {
			propertyKeyList = RegExpUtil.findAll(propertyKeys, "\\w+");
		}

		while (iterator.hasNext()) {
			OrgEntity next = iterator.next();
			OrgDomain d = new OrgDomain();
			list.add(d);

			d.setCode(next.getCode());
			d.setContacts(next.getContacts());
			d.setEnable(next.getEnable());
			d.setId(next.getId());
			d.setName(next.getName());
			d.setParentId(next.getParentId());
			d.setRemark(next.getRemark());
			d.setRootId(next.getRootId());
			d.setTelephone(next.getTelephone());
			d.setCreationTime(next.getCreationTime());
			d.setUpdateTime(next.getUpdateTime());

			if (CollectionUtils.isNotEmpty(propertyKeyList)) {
				Map<String, String> properties = getProperties(d.getId(), propertyKeyList);
				d.setProperties(properties);
			}
		}

		PageInfo<OrgDomain> ret = new PageInfo<OrgDomain>();
		ret.setList(list);
		ret.setTotal(page.getTotalElements());
		return ret;
	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param enable
	 * @return
	 */
	@ApiOperation(value = "查询顶级机构")
	@GetMapping("getRootOrgList")
	public List<OrgEntity> getRootOrgList(@RequestParam(required = false) Boolean enable) {

		Specification<OrgEntity> specification = (root, query, cb) -> {
			List<Predicate> predicates = new ArrayList<>();
			predicates.add(cb.isNull(root.get("parentId")));
			if (null != enable) {
				predicates.add(cb.equal(root.get("enable"), enable));
			}
			return cb.and(predicates.toArray(new Predicate[predicates.size()]));
		};

		List<OrgEntity> ret = orgRepo.findAll(specification);
		return ret;
	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param parentId
	 * @param orgName
	 * @return
	 */
	@ApiOperation(value = "查询一级子机构列表")
	@GetMapping("query")
	public List<OrgEntity> querySubOrgList(@RequestParam(required = true) String name,
			@RequestParam(required = false) Boolean enable) {

		if (StringUtils.isBlank(name)) {
			List<OrgEntity> list = Lists.newArrayList();
			return list;
		}

		Specification<OrgEntity> specification = (root, query, cb) -> {
			List<Predicate> predicates = new ArrayList<>();
			predicates.add(cb.equal(root.get("rootId"), getRootOrgId()));
			if (StringUtils.isNotBlank(name)) {
				predicates.add(cb.like(root.get("name"), toSqlSearchPattern(name)));
			}
			if (null != enable) {
				predicates.add(cb.equal(root.get("enable"), enable));
			}

			return cb.and(predicates.toArray(new Predicate[predicates.size()]));
		};

		PageRequest pageRequest = new PageRequest(0, 50, new Sort(Direction.DESC, "updateTime"));
		Page<OrgEntity> page = orgRepo.findAll(specification, pageRequest);
		Iterator<OrgEntity> iterator = page.iterator();
		List<OrgEntity> list = Lists.newArrayList();
		while (iterator.hasNext()) {
			OrgEntity next = iterator.next();

			OrgEntity bean = new OrgEntity();
			bean.setCode(next.getCode());
			bean.setContacts(next.getContacts());
			bean.setEnable(bean.getEnable());
			bean.setId(next.getId());
			bean.setName(next.getName());
			bean.setParentId(next.getParentId());
			bean.setRemark(next.getRemark());
			bean.setRootId(next.getRootId());
			bean.setTelephone(next.getTelephone());

			list.add(bean);
		}

		return list;
	}

	@ApiOperation(value = "查询全量机构树")
	@GetMapping("queryFullOrgTree")
	public List<OrgDomain> queryFullOrgTree(@RequestParam(required = false) String propertyKeys) {
		User accessUser = getAccessUser();
		Long rootOrgId = accessUser.getRootOrgId();

		Specification<OrgEntity> specification = (root, query, cb) -> {
			List<Predicate> predicates = new ArrayList<>();
			predicates.add(cb.equal(root.get("rootId"), rootOrgId));

			return cb.and(predicates.toArray(new Predicate[predicates.size()]));
		};

		// 过载保护
		long total = orgRepo.count(specification);
		if (total > 1000) {
			throw new StatusException("B-001251", "total is larger than 1000");
		}

		Sort sort = new Sort(Direction.ASC, "id");
		List<OrgEntity> orgEntityList = orgRepo.findAll(specification, sort);

		List<OrgDomain> ret = Lists.newArrayList();
		Iterator<OrgEntity> iterator = orgEntityList.iterator();

		List<String> propertyKeyList = null;
		if (StringUtils.isNotBlank(propertyKeys)) {
			propertyKeyList = RegExpUtil.findAll(propertyKeys, "\\w+");
		}

		while (iterator.hasNext()) {
			OrgEntity next = iterator.next();
			OrgDomain d = new OrgDomain();
			ret.add(d);

			d.setCode(next.getCode());
			d.setContacts(next.getContacts());
			d.setEnable(next.getEnable());
			d.setId(next.getId());
			d.setName(next.getName());
			d.setParentId(next.getParentId());
			d.setRemark(next.getRemark());
			d.setRootId(next.getRootId());
			d.setTelephone(next.getTelephone());
			d.setCreationTime(next.getCreationTime());
			d.setUpdateTime(next.getUpdateTime());

			if (CollectionUtils.isNotEmpty(propertyKeyList)) {
				Map<String, String> properties = getProperties(d.getId(), propertyKeyList);
				d.setProperties(properties);
			}
		}

		return ret;
	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param orgId
	 * @return
	 */
	@ApiOperation(value = "查询机构所有属性")
	@GetMapping("allProperties/{orgId}")
	public Map<String, String> getAllOrgProperties(@PathVariable Long orgId) {

		OrgEntity orgEntity = orgRepo.findOne(orgId);
		if (null == orgEntity) {
			throw new StatusException("B-001250", "orgId is wrong");
		}
		validateRootOrgIsolation(orgEntity.getRootId());

		Map<String, String> map = Maps.newHashMap();
		List<OrgPropertyEntity> list = orgPropertyRepo.findByOrgId(orgId);
		DynamicEnumManager manager = OrgProperty.getDynamicEnumManager();
		for (OrgPropertyEntity cur : list) {
			DynamicEnum de = manager.getById(cur.getKeyId());
			map.put(de.getName(), cur.getValue());
		}

		return map;
	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param orgId
	 * @param key
	 * @return
	 */
	@ApiOperation(value = "查询机构单个属性")
	@GetMapping("property/{orgId}/{key}")
	public String getOrgProperty(@PathVariable Long orgId, @PathVariable String key) {
		OrgEntity orgEntity = orgRepo.findOne(orgId);
		if (null == orgEntity) {
			throw new StatusException("B-001250", "orgId is wrong");
		}
		validateRootOrgIsolation(orgEntity.getRootId());

		DynamicEnumManager manager = OrgProperty.getDynamicEnumManager();
		DynamicEnum de = manager.getByName(key);
		OrgPropertyEntity one = orgPropertyRepo.findByOrgIdAndKeyId(orgId, de.getId());
		if (null == one) {
			return null;
		}
		return one.getValue();
	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param domain
	 * @return
	 */
	@ApiOperation(value = "新增顶级机构", notes = "新增")
	@PostMapping("addRootOrg")
	public OrgEntity addRootOrg(@RequestBody OrgDomain domain) {
		trim(domain, true);

		if (!isSuperAdmin()) {
			throw new StatusException("B-140001", "非法访问");
		}

		if (StringUtils.isBlank(domain.getCode())) {
			throw new StatusException("B-150001", "code is null");
		}

		OrgEntity orgEntity = orgRepo.findByParentIdIsNullAndCode(domain.getCode());
		if (null != orgEntity) {
			throw new StatusException("B-150001", "机构代码已存在");
		}

		OrgInfo info = new OrgInfo();
		info.setCode(domain.getCode());
		info.setName(domain.getName());
		info.setContacts(domain.getContacts());
		info.setTelephone(domain.getTelephone());
		info.setDomainName(domain.getDomainName());
		info.setRemark(domain.getRemark());

		Map<String, String> properties = domain.getProperties();
		if (null == properties) {
			properties = Maps.newHashMap();
		}
		info.setProperties(properties);

		OrgEntity saved = orgService.saveRootOrg(info);
		return saved;
	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param domain
	 * @return
	 */
	@ApiOperation(value = "更新顶级机构", notes = "更新")
	@PutMapping("updateRootOrg")
	public OrgEntity updateRootOrg(@RequestBody OrgDomain domain) {
		trim(domain, true);

		if (!isSuperAdmin()) {
			throw new StatusException("B-140001", "非法访问");
		}

		Long id = domain.getId();
		if (null == id) {
			throw new StatusException("B-140001", "id is null");
		}

		OrgEntity orgEntity = orgRepo.findOne(id);
		if (null == orgEntity) {
			throw new StatusException("B-140002", "orgEntity is null");
		}

		validateRootOrgIsolation(orgEntity.getRootId());

		OrgInfo info = new OrgInfo();

		info.setCode(orgEntity.getCode());
		info.setParentId(orgEntity.getParentId());
		info.setRootId(orgEntity.getRootId());

		info.setName(domain.getName());
		info.setContacts(domain.getContacts());
		info.setTelephone(domain.getTelephone());
		info.setDomainName(domain.getDomainName());
		info.setRemark(domain.getRemark());

		Map<String, String> properties = domain.getProperties();
		if (null == properties) {
			properties = Maps.newHashMap();
		}
		info.setProperties(properties);

		OrgEntity saved = orgService.saveRootOrg(info);
		return saved;
	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param domain
	 * @return
	 */
	@ApiOperation(value = "新增子机构", notes = "新增")
	@PostMapping("addSubOrg")
	public OrgEntity addSubOrg(@RequestBody OrgDomain domain) {
		trim(domain, true);

		User accessUser = getAccessUser();
		Long rootId = domain.getRootId();
		if (null == rootId) {
			rootId = accessUser.getRootOrgId();
		}

		validateRootOrgIsolation(rootId);

		OrgEntity subOrg = orgRepo.findByRootIdAndCode(accessUser.getRootOrgId(), domain.getCode());
		if (null != subOrg) {
			throw new StatusException("B-140001", "机构代码已存在");
		}

		Long parentId = domain.getParentId();
		if (null == parentId) {
			parentId = rootId;
		}

		OrgInfo info = new OrgInfo();
		info.setCode(domain.getCode());
		info.setName(domain.getName());
		info.setContacts(domain.getContacts());
		info.setTelephone(domain.getTelephone());
		info.setParentId(parentId);
		info.setRootId(rootId);
		info.setRemark(domain.getRemark());
		info.setEnable(domain.getEnable());

		Map<String, String> properties = domain.getProperties();
		if (null == properties) {
			properties = Maps.newHashMap();
		}
		info.setProperties(properties);

		OrgEntity saved = orgService.saveSubOrg(info);
		return saved;
	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param domain
	 * @return
	 */
	@ApiOperation(value = "更新子机构", notes = "更新")
	@PutMapping("updateSubOrg")
	public OrgEntity updateSubOrg(@RequestBody OrgDomain domain) {
		trim(domain, true);

		Long id = domain.getId();
		if (null == id) {
			throw new StatusException("B-140001", "id is null");
		}

		OrgEntity orgEntity = orgRepo.findOne(id);
		if (null == orgEntity) {
			throw new StatusException("B-140002", "orgEntity is null");
		}

		validateRootOrgIsolation(orgEntity.getRootId());

		OrgInfo info = new OrgInfo();
		info.setCode(orgEntity.getCode());
		info.setParentId(orgEntity.getParentId());
		info.setRootId(orgEntity.getRootId());

		info.setName(domain.getName());
		info.setContacts(domain.getContacts());
		info.setTelephone(domain.getTelephone());
		info.setRemark(domain.getRemark());
		info.setEnable(domain.getEnable());

		Map<String, String> properties = domain.getProperties();
		if (null == properties) {
			properties = Maps.newHashMap();
		}
		info.setProperties(properties);

		OrgEntity saved = orgService.saveSubOrg(info);
		return saved;
	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param ids
	 */
	@ApiOperation(value = "启用机构", notes = "启用")
	@PutMapping("enable/{ids}")
	public void enableOrgs(@PathVariable String ids) {
		List<Long> orgIds = Stream.of(ids.split(",")).map(s -> Long.parseLong(s.trim()))
				.collect(Collectors.toList());
		for (Long orgId : orgIds) {
			OrgEntity org = orgRepo.findOne(orgId);
			org.setEnable(true);
			orgRepo.save(org);
		}
	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param ids
	 */
	@ApiOperation(value = "禁用机构", notes = "禁用")
	@PutMapping("disable/{ids}")
	public void disableOrgs(@PathVariable String ids) {
		List<Long> orgIds = Stream.of(ids.split(",")).map(s -> Long.parseLong(s.trim()))
				.collect(Collectors.toList());
		for (Long orgId : orgIds) {
			OrgEntity org = orgRepo.findOne(orgId);
			org.setEnable(false);
			orgRepo.save(org);
		}
	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param domain
	 * @throws IOException
	 */
	@ApiOperation(value = "获取logo")
	@GetMapping("logo")
	public void getLogo(@RequestParam("domain") String domain, HttpServletResponse response)
			throws IOException {
		OrgEntity org = orgRepo.findByParentIdIsNullAndDomainName(domain);
		if (null == org) {
			throw new StatusException("B-140002", "orgEntity is null");
		}

		DynamicEnumManager manager = OrgProperty.getDynamicEnumManager();
		DynamicEnum de = manager.getByName("LOGO_PATH");

		OrgPropertyEntity entity = orgPropertyRepo.findByOrgIdAndKeyId(org.getId(), de.getId());
		if (null != entity) {
			IOUtils.copy(new FileInputStream(entity.getValue()), response.getOutputStream());
			response.flushBuffer();
		} else {
			throw new StatusException("B-140003", "no logo");
		}
	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param id
	 * @param request
	 * @param file
	 * @throws IOException
	 */
	@ApiOperation(value = "导入logo", notes = "导入logo")
	@PostMapping("importLogo/{orgId}")
	public void importLogo(@PathVariable Long orgId, HttpServletRequest request,
			@RequestParam CommonsMultipartFile file) throws IOException {

		OrgEntity orgEntity = orgRepo.findOne(orgId);
		if (null == orgEntity) {
			throw new StatusException("B-140002", "orgEntity is null");
		}

		validateRootOrgIsolation(orgEntity.getRootId());

		String schoolLogoPath = PropertiesUtil.getString(PropKeys.SCHOOL_LOGO_PATH);
		if (StringUtils.isBlank(schoolLogoPath)) {
			FileUtils.forceMkdir(new File(schoolLogoPath));
		}
		FileItem fileItem = file.getFileItem();
		String name = fileItem.getName();
		String filePath = null;
		if (name.endsWith(".jpg")) {
			filePath = schoolLogoPath + "/" + orgId + ".jpg";
		} else if (name.endsWith(".gif")) {
			filePath = schoolLogoPath + "/" + orgId + ".gif";
		} else if (name.endsWith(".png")) {
			filePath = schoolLogoPath + "/" + orgId + ".png";
		} else {
			throw new StatusException("B-101001", "文件格式错误");
		}

		FileUtils.copyInputStreamToFile(file.getInputStream(), new File(filePath));

		DynamicEnumManager manager = OrgProperty.getDynamicEnumManager();
		DynamicEnum de = manager.getByName("LOGO_PATH");

		OrgPropertyEntity entity = orgPropertyRepo.findByOrgIdAndKeyId(orgId, de.getId());
		if (null == entity) {
			entity = new OrgPropertyEntity();
			entity.setKeyId(de.getId());
			entity.setOrgId(orgId);
		}
		entity.setValue(filePath);
		orgPropertyRepo.save(entity);
	}

	@ApiOperation(value = "删除子机构", notes = "")
	@DeleteMapping("deleteSubOrg/{orgId}")
	public void deleteSubOrg(@PathVariable Long orgId) {
		orgService.deleteSubOrg(orgId, false);
	}

	@ApiOperation(value = "下载导入模板", notes = "下载导入模板")
	@GetMapping("importTemplate")
	public void getDownloadTemplate(HttpServletResponse response) {
		String resoucePath = PathUtil.getResoucePath("templates/subOrgImportTemplate.xlsx");
		exportFile("学习中心导入模板.xlsx", new File(resoucePath));
	}

	@ApiOperation(value = "导入子机构", notes = "")
	@PostMapping("importSubOrg")
	public Map<String, Object> importSubOrg(@RequestParam CommonsMultipartFile file) {
		DiskFileItem item = (DiskFileItem) file.getFileItem();
		File storeLocation = item.getStoreLocation();
		List<Map<String, Object>> failRecords = orgService.importSubOrg(getRootOrgId(),
				storeLocation);
		Map<String, Object> map = Maps.newHashMap();
		map.put("hasError", CollectionUtils.isNotEmpty(failRecords));
		map.put("failRecords", failRecords);
		return map;
	}

}
