package cn.com.qmth.examcloud.core.basic.api.controller;

import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import javax.persistence.criteria.Predicate;
import javax.persistence.criteria.Root;
import javax.persistence.criteria.Subquery;

import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringEscapeUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.data.domain.Sort;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.google.common.collect.Lists;
import com.google.common.collect.Maps;

import cn.com.qmth.examcloud.commons.base.exception.StatusException;
import cn.com.qmth.examcloud.commons.web.helpers.page.PageInfo;
import cn.com.qmth.examcloud.commons.web.security.bean.User;
import cn.com.qmth.examcloud.commons.web.security.enums.RoleMeta;
import cn.com.qmth.examcloud.commons.web.support.ControllerSupport;
import cn.com.qmth.examcloud.core.basic.api.controller.bean.UserDomain;
import cn.com.qmth.examcloud.core.basic.api.controller.bean.UserFormDomain;
import cn.com.qmth.examcloud.core.basic.base.constants.BasicConsts;
import cn.com.qmth.examcloud.core.basic.base.enums.AccountType;
import cn.com.qmth.examcloud.core.basic.dao.OrgRepo;
import cn.com.qmth.examcloud.core.basic.dao.RoleRepo;
import cn.com.qmth.examcloud.core.basic.dao.UserRepo;
import cn.com.qmth.examcloud.core.basic.dao.UserRoleRelationRepo;
import cn.com.qmth.examcloud.core.basic.dao.entity.OrgEntity;
import cn.com.qmth.examcloud.core.basic.dao.entity.RoleEntity;
import cn.com.qmth.examcloud.core.basic.dao.entity.UserEntity;
import cn.com.qmth.examcloud.core.basic.dao.entity.UserRoleRelationEntity;
import cn.com.qmth.examcloud.core.basic.service.AuthService;
import cn.com.qmth.examcloud.core.basic.service.bean.LoginInfo;
import cn.com.qmth.examcloud.core.basic.service.impl.UserServiceImpl;
import io.swagger.annotations.ApiOperation;

/**
 * 用户服务API Created by songyue on 17/1/13.
 */
@Transactional
@RestController
@RequestMapping("${$rmp.ctr.basic}/user")
public class UserController extends ControllerSupport {

	@Autowired
	UserServiceImpl userService;

	@Autowired
	UserRepo userRepo;

	@Autowired
	OrgRepo orgRepo;

	@Autowired
	RoleRepo roleRepo;

	@Autowired
	AuthService authService;

	@Autowired
	UserRoleRelationRepo userRoleRelationRepo;

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param curPage
	 * @param pageSize
	 * @param rootOrgId
	 * @param loginName
	 * @param name
	 * @param enable
	 * @param roleId
	 * @param roleCode
	 * @return
	 */
	@ApiOperation(value = "查询所有用户", notes = "")
	@GetMapping("all/{curPage}/{pageSize}")
	public PageInfo<UserDomain> getUserPage(@PathVariable Integer curPage,
			@PathVariable Integer pageSize, @RequestParam(required = false) Long rootOrgId,
			@RequestParam String loginName, @RequestParam String name,
			@RequestParam(required = false) Boolean enable,
			@RequestParam(required = false) Long roleId, @RequestParam(required = false) Long orgId,
			@RequestParam(required = false) String roleCode) {

		User accessUser = getAccessUser();

		if (null == rootOrgId) {
			rootOrgId = accessUser.getRootOrgId();
		} else {
			validateRootOrgIsolation(rootOrgId);
		}

		final Long finalRootOrgId = rootOrgId;

		OrgEntity rootOrg = orgRepo.findOne(rootOrgId);
		if (null == rootOrg) {
			throw new StatusException("B-150003", "机构不存在");
		}
		if (null != rootOrg.getParentId()) {
			throw new StatusException("B-150004", "机构错误");
		}

		if (null != roleId) {
			RoleEntity roleEntity = roleRepo.findOne(roleId);
			if (null == roleEntity) {
				throw new StatusException("B-150002", "角色不存在");
			}
		} else if (StringUtils.isNotBlank(roleCode)) {
			RoleEntity roleEntity = roleRepo.findByCodeAndRootOrgIdIsNull(roleCode.trim());
			if (null == roleEntity) {
				roleEntity = roleRepo.findByCodeAndRootOrgId(roleCode.trim(),
						accessUser.getRootOrgId());
			}
			if (null == roleEntity) {
				throw new StatusException("B-150002", "角色不存在");
			}
			roleId = roleEntity.getId();
		}

		final Long finalRoleId = roleId;

		Specification<UserEntity> specification = (root, query, cb) -> {
			List<Predicate> predicates = new ArrayList<>();
			predicates.add(cb.equal(root.get("rootOrgId"), finalRootOrgId));

			if (StringUtils.isNotBlank(loginName)) {
				predicates.add(cb.like(root.get("loginName"), toSqlSearchPattern(loginName)));
			}
			if (StringUtils.isNotBlank(name)) {
				predicates.add(cb.like(root.get("name"), toSqlSearchPattern(name)));
			}
			if (null != orgId) {
				predicates.add(cb.equal(root.get("orgId"), orgId));
			}
			if (null != finalRoleId) {
				Subquery<UserRoleRelationEntity> subquery = query
						.subquery(UserRoleRelationEntity.class);
				Root<UserRoleRelationEntity> subRoot = subquery.from(UserRoleRelationEntity.class);
				subquery.select(subRoot.get("userId"));
				Predicate p1 = cb.equal(subRoot.get("roleId"), finalRoleId);
				Predicate p2 = cb.equal(subRoot.get("userId"), root.get("id"));
				subquery.where(cb.and(p1, p2));
				predicates.add(cb.exists(subquery));
			}
			return cb.and(predicates.toArray(new Predicate[predicates.size()]));
		};
		Pageable pageable = new PageRequest(curPage - 1, pageSize, Sort.Direction.DESC,
				"updateTime");

		Page<UserEntity> userList = userRepo.findAll(specification, pageable);

		Iterator<UserEntity> iterator = userList.iterator();

		List<UserDomain> fullUserInfoList = Lists.newArrayList();
		while (iterator.hasNext()) {
			UserEntity next = iterator.next();
			UserDomain bean = new UserDomain();
			bean.setId(next.getId());
			bean.setLoginName(next.getLoginName());
			bean.setName(next.getName());
			bean.setRootOrgId(next.getRootOrgId());
			bean.setUpdateTime(next.getUpdateTime());
			bean.setCreationTime(next.getCreationTime());
			bean.setOrgId(next.getOrgId());
			if (null != bean.getOrgId()) {
				OrgEntity org = orgRepo.findOne(Long.valueOf(bean.getOrgId()));
				if (null != org) {
					bean.setOrgName(org.getName());
				}
			}
			bean.setRootOrgName(rootOrg.getName());
			bean.setEnable(next.getEnable());

			List<UserRoleRelationEntity> relationList = userRoleRelationRepo
					.findAllByUserId(next.getId());
			List<String> roleNameList = Lists.newArrayList();
			List<Long> roleIdList = Lists.newArrayList();
			List<String> roleCodeList = Lists.newArrayList();
			for (UserRoleRelationEntity cur : relationList) {
				RoleEntity curRoleEntity = roleRepo.findOne(cur.getRoleId());
				if (null == curRoleEntity) {
					throw new StatusException("B-150002", "角色错误");
				}
				roleNameList.add(curRoleEntity.getName());
				roleIdList.add(curRoleEntity.getId());
				roleCodeList.add(curRoleEntity.getCode());
			}
			bean.setRoleNamesStr(StringUtils.join(roleNameList, ","));
			bean.setRoleIds(roleIdList);
			bean.setRoleCodes(roleCodeList);

			fullUserInfoList.add(bean);
		}

		PageInfo<UserDomain> ret = new PageInfo<UserDomain>();
		ret.setList(fullUserInfoList);
		ret.setTotal(userList.getTotalElements());
		return ret;
	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param id
	 * @return
	 */
	@ApiOperation(value = "按id查询用户", notes = "id查询")
	@GetMapping("/{id}")
	public UserEntity getUserById(@PathVariable long id) {
		UserEntity user = userRepo.findOne(id);
		return user;
	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param orgId
	 * @return
	 */
	@ApiOperation(value = "按orgId查询用户", notes = "机构id查询机构用户")
	@GetMapping("/org/{orgId}")
	public List<UserEntity> getUserByOrgId(@PathVariable long orgId) {
		List<UserEntity> userList = userRepo.findByOrgId(orgId);
		return userList;
	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param rootOrgId
	 * @return
	 */
	@ApiOperation(value = "按rootOrgId查询用户", notes = "根机构id查询机构用户")
	@GetMapping("/rootOrg/{rootOrgId}")
	public List<UserEntity> getUserByRootOrgId(@PathVariable long rootOrgId) {
		List<UserEntity> userList = userRepo.findByRootOrgId(rootOrgId);
		return userList;
	}

	/**
	 * 重构 2018年6月26日
	 *
	 * @author WANGWEI
	 * @param userForm
	 * @return
	 */
	@ApiOperation(value = "新增用户", notes = "新增")
	@PostMapping
	public Map<String, Object> addUser(@RequestBody UserFormDomain userForm) {
		trim(userForm);
		userForm.setId(null);
		return saveUser(userForm);
	}

	/**
	 * 重构 2018年6月26日
	 *
	 * @author WANGWEI
	 * @param userForm
	 * @return
	 */
	@ApiOperation(value = "更新用户", notes = "更新")
	@PutMapping
	public Map<String, Object> updateUser(@RequestBody UserFormDomain userForm) {
		trim(userForm);
		if (null == userForm.getId()) {
			throw new StatusException("B-150009", "user ID is null");
		}

		UserEntity userEntity = userRepo.findOne(userForm.getId());
		if (null == userEntity) {
			throw new StatusException("B-150010", "用户不存在");
		}

		if (!userEntity.getRootOrgId().equals(userForm.getRootOrgId())) {
			throw new StatusException("B-150010", "顶级机构错误");
		}

		userForm.setPassword(userEntity.getPassword());

		return saveUser(userForm);
	}

	/**
	 * 新增或更新用户
	 *
	 * @author WANGWEI
	 * @param userForm
	 * @return
	 */
	private Map<String, Object> saveUser(UserFormDomain userForm) {
		User accessUser = getAccessUser();
		Long rootOrgId = userForm.getRootOrgId();
		OrgEntity rootOrg = orgRepo.findOne(rootOrgId);
		if (null == rootOrg) {
			throw new StatusException("B-150003", "机构不存在");
		}
		if (null != rootOrg.getParentId()) {
			throw new StatusException("B-150004", "机构错误");
		}

		if ((!isSuperAdmin()) && (!rootOrgId.equals(accessUser.getRootOrgId()))) {
			throw new StatusException("B-150005", "无权操作");
		}

		UserEntity userEntity = new UserEntity();
		userEntity.setId(userForm.getId());
		userEntity.setEnable(userForm.getEnable());
		userEntity.setLoginName(userForm.getLoginName());
		userEntity.setPhoneNumber(userForm.getMobile());
		userEntity.setName(userForm.getName());
		userEntity.setOrgId(userForm.getOrgId());
		userEntity.setRootOrgId(rootOrgId);
		userEntity.setPassword(userForm.getPassword());
		userEntity.setUpdateTime(new Date());

		UserEntity saved = userService.save(userEntity);

		List<UserRoleRelationEntity> userRoles = Lists.newArrayList();
		List<Long> roleIds = userForm.getRoleIds();
		for (Long cur : roleIds) {
			RoleEntity curRoleEntity = roleRepo.findOne(cur);
			if (null == curRoleEntity) {
				throw new StatusException("B-150006", "角色错误");
			}
			if (curRoleEntity.getCode().equals(RoleMeta.SUPER_ADMIN.name())) {
				throw new StatusException("B-150007", "不允许新增或修改超级管理员");
			}
			UserRoleRelationEntity relation = new UserRoleRelationEntity(saved.getId(),
					curRoleEntity.getId());
			userRoles.add(relation);
		}

		userRoleRelationRepo.deleteByUserId(saved.getId());
		List<UserRoleRelationEntity> savedRelationList = userRoleRelationRepo.save(userRoles);

		Map<String, Object> ret = Maps.newHashMap();
		ret.put("userId", saved.getId());
		ret.put("loginName", saved.getLoginName());
		ret.put("roleList", savedRelationList);
		return ret;
	}

	/**
	 * 判断是否是超级管理员
	 *
	 * @author WANGWEI
	 * @param userId
	 * @return
	 */
	private boolean isSuperAdmin(Long userId) {
		List<UserRoleRelationEntity> relationList = userRoleRelationRepo.findAllByUserId(userId);
		if (CollectionUtils.isNotEmpty(relationList)) {
			for (UserRoleRelationEntity cur : relationList) {
				Long roleId = cur.getRoleId();
				RoleEntity roleEntity = roleRepo.findOne(roleId);
				if (roleEntity.getCode().equals(RoleMeta.SUPER_ADMIN.name())) {
					return true;
				}
			}
		}

		return false;
	}

	@ApiOperation(value = "重置用户密码", notes = "重置密码")
	@PutMapping("/resetPass/{id}")
	public void resetPass(@PathVariable String id) {
		List<Long> ids = Stream.of(id.split(",")).map(s -> Long.parseLong(s.trim()))
				.collect(Collectors.toList());

		for (Long userId : ids) {
			if (isSuperAdmin(userId)) {
				throw new StatusException("B-150410", "超级管理员账号不允许修改");
			}
			UserEntity user = userRepo.findOne(userId);
			user.setPassword(BasicConsts.DEFAULT_PASSWORD);
			userRepo.save(user);
		}
	}

	@ApiOperation(value = "启用用户", notes = "启用用户")
	@PutMapping("/enable/{ids}")
	public List<String> enableUser(@PathVariable String ids) {
		List<Long> userIds = Stream.of(ids.split(",")).map(s -> Long.parseLong(s.trim()))
				.collect(Collectors.toList());
		List<String> ret = Lists.newArrayList();
		for (Long userId : userIds) {
			if (isSuperAdmin(userId)) {
				throw new StatusException("B-150410", "超级管理员账号不允许修改");
			}
			UserEntity user = userRepo.findOne(userId);
			user.setEnable(true);
			userRepo.save(user);
			ret.add(user.getId() + ":" + user.getName());
		}
		return ret;
	}

	@ApiOperation(value = "禁用用户", notes = "禁用用户")
	@PutMapping("/disable/{ids}")
	public List<String> disableUser(@PathVariable String ids) {
		List<Long> userIds = Stream.of(ids.split(",")).map(s -> Long.parseLong(s.trim()))
				.collect(Collectors.toList());
		List<String> ret = Lists.newArrayList();
		for (Long userId : userIds) {
			if (isSuperAdmin(userId)) {
				throw new StatusException("B-150410", "超级管理员账号不允许修改");
			}
			UserEntity user = userRepo.findOne(userId);
			user.setEnable(false);
			userRepo.save(user);
			ret.add(user.getId() + ":" + user.getName());
		}
		return ret;
	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param userId
	 * @param password
	 */
	@ApiOperation(value = "修改用户密码", notes = "修改密码")
	@PutMapping("/password")
	public void updatePass(@RequestParam long userId, @RequestParam String password) {
		String realPassword = StringEscapeUtils.unescapeJavaScript(password);
		userRepo.updatePasswordById(userId, realPassword);
	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param ids
	 */
	@ApiOperation(value = "按id删除用户", notes = "删除")
	@DeleteMapping("/{ids}")
	public void deleteUser(@PathVariable String ids) {
		List<Long> userIds = Stream.of(ids.split(",")).map(s -> Long.parseLong(s.trim()))
				.collect(Collectors.toList());
		for (Long userId : userIds) {
			userRepo.delete(userId);
		}
	}

	@ApiOperation(value = "一般登录", notes = "登录")
	@PostMapping("/login")
	public Map<String, Object> login(@RequestParam String loginName,
			@RequestParam String password) {
		UserEntity user = userRepo.findByLoginName(loginName);
		if (user == null) {
			throw new RuntimeException("该用户不存在");
		} else if (!user.getEnable()) {
			throw new RuntimeException("该用户被禁用");
		}

		LoginInfo loginInfo = new LoginInfo();
		loginInfo.setAccountType(AccountType.COMMON_LOGIN_NAME.name());
		loginInfo.setAccountValue(user.getLoginName());
		loginInfo.setPassword(user.getPassword());
		loginInfo.setRootOrgId(user.getRootOrgId());
		User loginUser = authService.login(loginInfo);

		Map<String, Object> ret = Maps.newHashMap();
		ret.put("userId", loginUser.getUserId());
		ret.put("token", loginUser.getKey() + ":" + loginUser.getToken());
		return ret;
	}

	@ApiOperation(value = "二级登录", notes = "二级登录")
	@PostMapping("/login/{rootOrgId}")
	public Map<String, Object> login(@PathVariable long rootOrgId, @RequestParam String loginName,
			@RequestParam String password) {
		UserEntity user = userRepo.findByRootOrgIdAndLoginName(rootOrgId, loginName);
		if (user == null) {
			throw new RuntimeException("该用户不存在");
		} else if (!user.getEnable()) {
			throw new RuntimeException("该用户被禁用");
		}

		LoginInfo loginInfo = new LoginInfo();
		loginInfo.setAccountType(AccountType.COMMON_LOGIN_NAME.name());
		loginInfo.setAccountValue(user.getLoginName());
		loginInfo.setPassword(user.getPassword());
		loginInfo.setRootOrgId(user.getRootOrgId());
		User loginUser = authService.login(loginInfo);

		Map<String, Object> ret = Maps.newHashMap();
		ret.put("userId", loginUser.getUserId());
		ret.put("token", loginUser.getKey() + ":" + loginUser.getToken());
		return ret;
	}

}
